"use strict";
exports.__esModule = true;
var codeFrame = require("babel-code-frame");
var printers_1 = require("./printers");
var builtins = new Set([
    'Array',
    'Date',
    'Error',
    'Function',
    'Object',
    'Map',
    'Promise',
    'RegExp',
    'Set',
    'undefined',
]);
var Output = /** @class */ (function () {
    function Output(ctx) {
        this.imports = [];
        this.declarations = [];
        this.exports = [];
        this.printed = new Set();
        this.ctx = ctx;
    }
    Output.prototype.request = function (id) {
        if (this.printed.has(id.name)) {
            return;
        }
        this.printed.add(id.name);
        var declaration = this.ctx.declarations.get(id.name);
        if (!declaration) {
            if (builtins.has(id.name)) {
                return;
            }
            switch (id.name) {
                case 'tty$ReadStream':
                    this.imports.push("import {ReadStream as tty$ReadStream} from 'tty';");
                    return;
                case 'tty$WriteStream':
                    this.imports.push("import {WriteStream as tty$WriteStream} from 'tty';");
                    return;
            }
            throw this.ctx.getError('Unable to resolve the reference to ' +
                id.name +
                '. Maybe it should be a builtin?', id);
        }
        for (var _i = 0, declaration_1 = declaration; _i < declaration_1.length; _i++) {
            var d = declaration_1[_i];
            if (d.type === 'SingleImportDeclaration') {
                this.imports.push(printers_1["default"](d, this.ctx));
            }
            else {
                this.declarations.push(printers_1["default"](d, this.ctx));
            }
        }
    };
    Output.prototype.toString = function (options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.ctx.namedExports.forEach(function (e) {
            return _this.exports.push("export " + printers_1["default"](e, _this.ctx) + ";");
        });
        if (this.ctx.defaultExport) {
            this.exports.push("export " + (options.commonJsDefaultExport ? '=' : 'default') + " " + printers_1["default"](this.ctx.defaultExport, this.ctx) + ";");
        }
        return (this.imports.join('\n') +
            '\n\n' +
            this.declarations.join('\n') +
            '\n\n' +
            this.exports.join('\n') +
            '\n');
    };
    return Output;
}());
exports.Output = Output;
var Context = /** @class */ (function () {
    function Context(filename, src, chain) {
        if (chain === void 0) { chain = []; }
        this.declarations = new Map();
        this.defaultExport = null;
        this.namedExports = [];
        this.exportNames = new Set();
        this.filename = filename;
        this.src = src;
        this.chain = chain;
        this.parents = this.chain.slice(0, this.chain.length - 1).reverse();
        this.output = new Output(this);
    }
    Context.prototype.getError = function (msg, node) {
        return new Error(msg +
            '\n\n' +
            codeFrame(this.src, node.loc.start.line, node.loc.start.column, {
                highlightCode: true
            }));
    };
    Context.prototype.declare = function (id, declaration) {
        var d = this.declarations.get(id.name) || [];
        this.declarations.set(id.name, d);
        d.push(declaration);
    };
    Context.prototype.setDefaultExport = function (value) {
        this.defaultExport = value;
    };
    Context.prototype.addNamedExport = function (value) {
        this.exportNames.add(value.exported.name);
        this.namedExports.push(value);
    };
    return Context;
}());
exports.Context = Context;
exports["default"] = Context;
